#=======================================================================================================================
# Multilayer perceptron is given in a separate file since it is not available in the python version employed in
# the other files.
#=======================================================================================================================

import pandas as pd
from Comparing import compare_methods_h5py
import argparse
from utils.conf import Conf
from StatisticalAnalysis.statisticalAnalysis import statisticalAnalysis
import os
import json

from shallowmodels.classificationModelFactory import classificationModelFactory


blacklist = [["haarhog", "SVM"],
             ["haarhog", "KNN"],
             ["haralick", "SVM"],
             ["haralick", "KNN"],
             ["hog", "SVM"],
             ["hog", "KNN"]]

def statisticalComparison(outputPath, datasetPath, featureExtractors, modelClassifiers, measure, verbose= False):
    pathAux = outputPath + datasetPath[datasetPath.rfind("/"):]
    filePathAux = pathAux + "/results/kfold-comparison_bestClassifiers.csv"

    for model in featureExtractors:

        if verbose:
            print(model)

        featuresPath = pathAux + "/models/features-" + model[0] + ".hdf5"
        # db = h5py.File(featuresPath)
        # labels = db["image_ids"]
        labelEncoderPath = pathAux + "/models/le-" + model[0] + ".cpickle"

        #conf["label_encoder_path"][0:conf["label_encoder_path"].rfind(".")] + "-" + model[0] + ".cpickle"
        # le = cPickle.loads(open(labelEncoderPath).read())
        # labels = [le.transform([l.split(":")[0]])[0] for l in labels]
        # df1 = pd.DataFrame([np.append(x,y) for (x,y) in zip(db["features"],labels)])

        # df = pd.read_csv(featuresCSVPath)
        # data = df.ix[:, :-1].values
        factory =classificationModelFactory()
        listAlgorithms = []
        listParams = []
        listNiter = []
        listNames = []
        filePath = pathAux + "/results/StatisticalComparison_" + model[0] + ".txt"

        for classificationModel in modelClassifiers:
            combination = [model[0], classificationModel]
            if (combination in blacklist):
                print("The combination("+ model[0] + "-" + classificationModel + ") is not allowed")
            else:

                if verbose:
                    print classificationModel
                modelClas = factory.getClassificationModel(classificationModel)
                cMo = modelClas.getModel()
                params = modelClas.getParams()
                niter = modelClas.getNIterations()
                listAlgorithms.append(cMo)
                listParams.append(params)
                listNiter.append(niter)
                listNames.append(classificationModel)

        if os.path.exists(pathAux + "/results"):
            if not os.path.isfile(filePathAux):
                fileResults = open(filePathAux, "a")
                for j in range(listNiter[0]):
                    fileResults.write("," + str(j))

                fileResults.write("\n")
            else:
                fileResults = open(filePathAux, "a")

        else:
            os.makedirs(filePathAux[:filePathAux.rfind("/")])
            fileResults = open(filePathAux, "a")
            for j in range(listNiter[0]):
                fileResults.write(","+str(j))

            fileResults.write("\n")

        # listNames = modelClassifiers
        if verbose:
            print("-------------------------------------------------")
            print("Statistical Analysis")
            print("-------------------------------------------------")

        #listAlgorithms = [clfRF, clfSVC, clfKNN, clfLR, clfMLP]  # ,clfET
        #listParams = [param_distRF, param_distSVC, param_distKNN, param_distLR, param_distMLP]  # ,param_distET
        #listNames = ["RF", "SVM", "KNN", "LR", "MLP"]  # ,"ET"
        #Niteraciones de las clases [10, 10, 10, 5, 10]

        resultsAccuracy = compare_methods_h5py(model, featuresPath, labelEncoderPath, listAlgorithms, listParams, listNames,
                                               listNiter,measure , verbose, normalization=False)  # ,10

        dfAccuracy = pd.DataFrame.from_dict(resultsAccuracy, orient='index')
        KFoldComparisionPathAccuracy = pathAux + "/results/kfold-comparison_"+model[0] + ".csv"
        #KFoldComparisionPathAccuracy=conf["kfold_comparison"][0:conf["kfold_comparison"].rfind(".")] + "-" + model[0] + ".csv"
        if (not os.path.exists(KFoldComparisionPathAccuracy[:KFoldComparisionPathAccuracy.rfind("/")])):
            os.mkdir(KFoldComparisionPathAccuracy[:KFoldComparisionPathAccuracy.rfind("/")])
        dfAccuracy.to_csv(KFoldComparisionPathAccuracy)
        statisticalAnalysis(KFoldComparisionPathAccuracy,filePath, fileResults,verbose)
    fileResults.close()
    filePath2 = pathAux + "/results/StatisticalComparison_bestClassifiers.txt"
    fileResults2 = open(pathAux + "/results/bestExtractorClassifier.csv", "a")
    statisticalAnalysis(pathAux + "/results/kfold-comparison_bestClassifiers.csv", filePath2, fileResults2, verbose)
    fileResults2.close()

    file = open(pathAux + "/results/bestExtractorClassifier.csv")
    line = file.read()
    extractorClassifier = line.split(",")[0]
    extractor, classifier = extractorClassifier.split("_")
    for model in featureExtractors:
        if model[0]==extractor:
            if len(model)==1:
                parametros =""
            else:
                parametros=model[1]
            fileConfModel = open(pathAux + "/ConfModel.json","w+")
            ConfModel={
                    'featureExtractor': {'model': model[0], 'params': str(parametros)},
                    'classificationModel': classifier
                }

            with fileConfModel as outfile:
                json.dump(ConfModel, outfile)

    del resultsAccuracy, dfAccuracy
        # sys.stdout = sys.__stdout__

def __main__():
    ap = argparse.ArgumentParser()
    ap.add_argument("-c", "--conf", required=True, help="path to configuration file")
    args = vars(ap.parse_args())
    # load the configuration and label encoder
    conf = Conf(args["conf"])

    outputPath = conf["output_path"]
    datasetPath = conf["dataset_path"]
    featureExtractors = conf["feature_extractors"]
    modelClassifiers = conf["model_classifiers"]
    measure = conf["measure"]


    # statisticalComparison(conf)
    statisticalComparison(outputPath, datasetPath, featureExtractors, modelClassifiers, measure, False)

if __name__ == "__main__":
    __main__()


# USAGE
# python train.py --conf conf/flowers17.json

# import the necessary packages
from __future__ import print_function
from utils.conf import Conf
import argparse
import cPickle
import h5py
import shutil
import json

from sklearn.model_selection import RandomizedSearchCV
import shallowmodels.classificationModelFactory as cmf

def train(outputPath, datasetPath, trainingSize):
	datasetName = datasetPath[datasetPath.rfind("/"):]
	auxPath = outputPath + datasetName

	with open(auxPath + "/ConfModel.json") as json_file:
		data = json.load(json_file)
	extractor = data['featureExtractor']
	classifier = data['classificationModel']
	labelEncoderPath = auxPath + "/models/le-" + extractor["model"] + ".cpickle"
	le = cPickle.loads(open(labelEncoderPath).read())
	# open the database and split the data into their respective training and
	# testing splits
	print("[INFO] gathering train/test splits...")

	featuresPath = auxPath + "/models/features-" + extractor["model"] + ".hdf5"
	db = h5py.File(featuresPath)
	split = int(db["image_ids"].shape[0] * trainingSize)
	(trainData, trainLabels) = (db["features"][:split], db["image_ids"][:split])
	# (testData, testLabels) = (db["features"][split:], db["image_ids"][split:])
	# use the label encoder to encode the training and testing labels
	trainLabels = [le.transform([l.split(":")[0]])[0] for l in trainLabels]
	# define the grid of parameters to explore, then start the grid search where
	# we evaluate a Linear SVM for each value of C
	print("[INFO] tuning hyperparameters...")

	factory = cmf.classificationModelFactory()
	classifierModel = factory.getClassificationModel(classifier)
	model = RandomizedSearchCV(classifierModel.getModel(), param_distributions=classifierModel.getParams(),
							   n_iter=classifierModel.getNIterations())

	model.fit(trainData, trainLabels)
	print("[INFO] best hyperparameters: {}".format(model.best_params_))
	# dump classifier to file
	print("[INFO] dumping classifier...")
	f = open(auxPath + "/classifier_" + extractor["model"] + "_" + classifier + ".cpickle", "w")
	f.write(cPickle.dumps(model))
	f.close()
	# close the database
	db.close()

	print("Do you want to generate a web app to classify the images with the best combination? y/n")
	webapp = raw_input()
	with open(auxPath + "/ConfModel.json") as json_file:
		data = json.load(json_file)
	extractor = data['featureExtractor']
	classifier = data['classificationModel']
	if (webapp.upper() in ("YES", "Y")):
		shutil.copyfile(auxPath + "/ConfModel.json", "./frimcla/webApp/FlaskApp/ConfModel.json")
		shutil.copyfile(auxPath + "/classifier_" + extractor["model"] + "_" + classifier + ".cpickle",
						"./frimcla/webApp/FlaskApp/classifier_" + extractor["model"] + "_" + classifier + ".cpickle")
		shutil.copyfile(auxPath + "/models/le-" + extractor["model"] + ".cpickle",
						"./frimcla/webApp/FlaskApp/le-" + extractor["model"] + ".cpickle")
		shutil.make_archive(auxPath + "/webapp", 'zip', './frimcla/webApp')


def __main__():
	# construct the argument parser and parse the command line arguments
	ap = argparse.ArgumentParser()
	ap.add_argument("-c", "--conf", required=True, help="path to configuration file")
	args = vars(ap.parse_args())
	# load the configuration and label encoder
	conf = Conf(args["conf"])

	outputPath=conf["output_path"]
	datasetPath=conf["dataset_path"]
	trainingSize=conf["training_size"]
	train(outputPath, datasetPath, trainingSize)

if __name__ == "__main__":
    __main__()


import classificationModels as cM

class classificationModelFactory():

    def __init__(self):
        pass

    def getClassificationModel(self,modelText):
        if modelText == "RandomForest":
            return cM.RandomForest()
        elif modelText == "SVM":
            return cM.SVM()
        elif modelText == "KNN":
            return cM.KNN()
        elif modelText == "LogisticRegression":
            return cM.LogRegression()
        elif modelText == "MLP":
            return cM.MultiLayerPerceptron()
        elif modelText == "GradientBoost":
            return  cM.GradientBoost()
        elif modelText == "ExtraTrees":
            return  cM.ExtraTrees()



import pandas as pd
import numpy as np
from sklearn.cross_validation import train_test_split
from sklearn.model_selection import RandomizedSearchCV
from sklearn.metrics import accuracy_score
from sklearn.metrics import roc_auc_score
from sklearn.metrics import precision_score
from sklearn.metrics import recall_score
from sklearn.metrics import f1_score

from sklearn.model_selection import KFold
import h5py
import cPickle
from sklearn.externals.joblib import Parallel, delayed

def apply_algorithm(tuple):
    clf, params, name, n_iter,trainData, trainLabels,testData,testLabels = tuple
    print(name)
    if params is None:
        model = clf
    else:
        model = RandomizedSearchCV(clf, param_distributions=params, n_iter=n_iter)
    model.fit(trainData, trainLabels)
    predictions = model.predict(testData)
    return (name,accuracy_score(testLabels, predictions))

def compare_method(tuple):
    i, (train_index, test_index),data,labels,listAlgorithms,listParameters,listAlgorithmNames,listNiters,normalization = tuple
    print("Iteration " + str(i+1))
    results = {name: [] for name in listAlgorithmNames}
    trainData, testData = data[train_index], data[test_index]
    trainLabels, testLabels = labels[train_index], labels[test_index]

    tuple = [(clf, params, name, n_iter, trainData, trainLabels, testData, testLabels) for clf, params, name, n_iter in
             zip(listAlgorithms, listParameters, listAlgorithmNames, listNiters)]
    #p = ThreadPool(len(listAlgorithms))
    comparison = map(apply_algorithm, tuple)
    for (name, comp) in comparison:
        results[name].append(comp)
    print("Finished iteration " + str(i))
    return (i,results)

def compare_methods(dataset,listAlgorithms,listParameters,listAlgorithmNames,listNiters,normalization=False, verbose=False):

    # Loading dataset
    df = pd.read_csv(dataset)
    data = df.ix[:, :-1].values
    labels = df.ix[:, -1].values
    kf = KFold(n_splits=10,shuffle=True,random_state=42)
    resultsAccuracy = {name:[] for name in listAlgorithmNames}
    #resultsAUROC = {name: [] for name in listAlgorithmNames}
    resultsPrecision = {name: [] for name in listAlgorithmNames}
    resultsRecall = {name: [] for name in listAlgorithmNames}
    resultsFmeasure = {name: [] for name in listAlgorithmNames}
    # tuple = [(i,(train_index,test_index),data,labels,listAlgorithms,listParameters,listAlgorithmNames,listNiters,normalization) for i,(train_index,test_index) in enumerate(kf.split(data))]
    # comparison = map(compare_method, tuple)
    #
    # for i,result in comparison:
    #     for name in listAlgorithmNames:
    #         results[name].append(result[name])

    for i,(train_index,test_index) in enumerate(kf.split(data)):
        if verbose:
            print "Iteration " + str(i+1) + "/10"

        trainData , testData = data[train_index],data[test_index]
        trainLabels, testLabels = labels[train_index], labels[test_index]

        # Normalization
        if normalization:
            trainData = np.asarray(trainData).astype("float64")
            trainData -= np.mean(trainData, axis=0)
            trainData /= np.std(trainData, axis=0)
            testData = np.asarray(testData).astype("float64")
            testData -= np.mean(testData, axis=0)
            testData /= np.std(testData, axis=0)
        # tuple = [(clf,params,name,n_iter,trainData, trainLabels,testData,testLabels) for clf,params,name,n_iter in zip(listAlgorithms,listParameters,listAlgorithmNames,listNiters)]
        # p = Pool(len(listAlgorithms))
        # comparison = p.map(apply_algorithm, tuple)
        # for (name, comp) in comparison:
        #     results[name].append(comp)
        #
        for clf,params,name,n_iter in zip(listAlgorithms,listParameters,listAlgorithmNames,listNiters):
            if verbose:
                print(name)
            if params is None:
                model = clf
            else:
                model = RandomizedSearchCV(clf, param_distributions=params,n_iter=n_iter)
            model.fit(trainData, trainLabels)
            predictions = model.predict(testData)
            resultsAccuracy[name].append(accuracy_score(testLabels, predictions))
            #resultsAUROC[name].append(roc_auc_score(testLabels, predictions))
            resultsPrecision[name].append(precision_score(testLabels, predictions))
            resultsRecall[name].append(recall_score(testLabels, predictions))
            resultsFmeasure[name].append(f1_score(testLabels, predictions))

    return (resultsAccuracy,resultsPrecision,resultsRecall,resultsFmeasure)


def accuracyPrediction(clf, params, name, n_iter, trainData, testData, trainLabels, testLabels, measure = "accuracy", verbose=False):
    if verbose:
        print(name)
    if params is None:
        model = clf
    else:
        model = RandomizedSearchCV(clf, param_distributions=params, n_iter=n_iter)

    trainData = np.nan_to_num(trainData)
    testData = np.nan_to_num(testData)
    model.fit(trainData, trainLabels)
    predictions = model.predict(testData)
    # return (name, accuracy_score(testLabels, predictions))
    return (name, measure_score(testLabels,predictions, measure))


def compare_methods_h5py(model, featuresPath,labelEncoderPath,listAlgorithms,listParameters,listAlgorithmNames,
                         listNiters, measure, verbose=False, normalization=False):

    # Loading dataset
    db = h5py.File(featuresPath)
    labels = db["image_ids"]
    data = db["features"][()]
    fileAux = open(labelEncoderPath)
    le = cPickle.loads(fileAux.read())
    fileAux.close()
    labels = np.asarray([le.transform([l.split(":")[0]])[0] for l in labels])
    del le
    kf = KFold(n_splits=10,shuffle=False,random_state=42) #n_splits=10
    resultsAccuracy = {model[0]+ "_" +name:[] for name in listAlgorithmNames}

    for i,(train_index,test_index) in enumerate(kf.split(data)):
        if verbose:
            print "Iteration " + str(i)
        trainData , testData = data[train_index],data[test_index]
        trainLabels, testLabels = labels[train_index], labels[test_index]

        # Normalization
        if normalization:
            trainData = np.asarray(trainData).astype("float32")
            trainData -= np.mean(trainData, axis=0)
            trainData /= np.std(trainData, axis=0)
            testData = np.asarray(testData).astype("float32")
            testData -= np.mean(testData, axis=0)
            testData /= np.std(testData, axis=0)

        output = Parallel(n_jobs=-1)(delayed(accuracyPrediction)(clf, params, name, n_iter, trainData, testData, trainLabels, testLabels, measure,verbose)
                           for clf, params, name, n_iter in
                           zip(listAlgorithms, listParameters, listAlgorithmNames, listNiters))

        #for clf, params, name, n_iter in zip(listAlgorithms, listParameters, listAlgorithmNames, listNiters):
        for name, accuracy in output:
            resultsAccuracy[model[0]+ "_" + name].append(accuracy)
        del output
    return resultsAccuracy


def measure_score(testLabels, predictions, measure_name = "accuracy"):
    if measure_name == "accuracy":
        return accuracy_score(testLabels, predictions)
    elif measure_name == "f1":
        return f1_score(testLabels, predictions)
    elif measure_name == "recall":
        return recall_score(testLabels, predictions)
    elif measure_name == "precision":
        return precision_score(testLabels, predictions)
    elif measure_name == "auroc":
        return roc_auc_score(testLabels, predictions)
    else:
        pass


from flask import Flask, render_template,request
from werkzeug.utils import secure_filename
import json
import cPickle
import frimcla.extractor.extractor as e
import frimcla.utils.dataset as dataset
import numpy as np
import os


app = Flask(__name__)
app.config['SERVER_NAME'] = 'localhost:5000'

@app.route("/")
def main():
    prediction=""
    # Para enviar variables al html lo unico que hay que hacer es llamar a las variables con el nombre con el que iran en la pagina
    return render_template('index.html', **locals())

@app.route("/predict", methods = ["POST"])
def prediction():
    prediction =""
    f = request.files['file']
    f.save(secure_filename(f.filename))
    with open('ConfModel.json') as data:
        datos = json.load(data)

    #Tener cuidado con esto, cuando suba la nueva version de frimcla a pip hay que quitar el literal eval
    featureExtractor = [str(datos["featureExtractor"]["model"]), datos["featureExtractor"]["params"]]
    classificationModel = datos["classificationModel"]
    cPickleFile = "classifier_" + featureExtractor[0] + "_" + classificationModel + ".cpickle"
    labelEncoderPath = "le-" + featureExtractor[0] + ".cpickle"
    le = cPickle.loads(open(labelEncoderPath).read())
    model = cPickle.loads(open(cPickleFile).read())
    oe = e.Extractor(featureExtractor)
    (labels, images) = dataset.build_batch([f.filename], featureExtractor[0])
    features = oe.describe(images)
    for (label, vector) in zip(labels, features):
        prediction = model.predict(np.atleast_2d(vector))[0]
        prediction = le.inverse_transform(prediction)
        # print("[INFO] predicted: {}".format(prediction))
    os.remove(f.filename)
    return render_template('index.html', **locals())

if __name__ == "__main__":
    app.run()


# USAGE
# python index_features.py --conf conf/flowers17.json

# suppress any FutureWarning from Theano
from __future__ import print_function
# from mpi4py import MPI
import warnings
warnings.simplefilter(action="ignore", category=FutureWarning)

# import the necessary packages
from extractor.extractor import Extractor
from indexer.indexer import Indexer
from utils.conf import Conf
from utils import dataset
from sklearn.preprocessing import LabelEncoder
from imutils import paths
import argparse
import cPickle
import random
import os

from guppy import hpy
hp = hpy()


def extractFeatures(fE, batchSize, imagePaths, outputPath, datasetPath, le, verbose = False):
	# initialize the Overfeat extractor and the Overfeat indexer
	if verbose:
		print("[INFO] initializing network...")
	oe = Extractor(fE)
	featuresPath = outputPath + datasetPath[datasetPath.rfind("/"):] + \
				   "/models/features-" + fE[0] + ".hdf5"
	labelEncoderPath = featuresPath[:featuresPath.rfind("/")] + "/le-" + fE[0] + ".cpickle"
	directory = featuresPath[:featuresPath.rfind("/")]
	if (not os.path.exists(directory)):
		os.makedirs(directory)

	if not (os.path.isfile(featuresPath) and  os.path.isfile(labelEncoderPath)):
		oi = Indexer(featuresPath, estNumImages=len(imagePaths))
		if verbose:
			print("[INFO] starting feature extraction...")

		# loop over the image paths in batches
		for (i, paths) in enumerate(dataset.chunk(imagePaths, batchSize)):
			# load the set of images from disk and describe them
			(labels, images) = dataset.build_batch(paths, fE[0])
			features = oe.describe(images)
			# loop over each set of (label, vector) pair and add them to the indexer
			for (label, vector) in zip(labels, features):
				oi.add(label, vector)

			# check to see if progress should be displayed
			if i > 0 and verbose:
				oi._debug("processed {} images".format((i + 1) * batchSize, msgType="[PROGRESS]"))

		# finish the indexing process
		oi.finish()
		# dump the label encoder to file
		if verbose:
			print("[INFO] dumping labels to file...")

		# confPath["label_encoder_path"][0:confPath["label_encoder_path"].rfind(".")] + "-" + fE[0] + ".cpickle"
		f = open(labelEncoderPath, "w")
		f.write(cPickle.dumps(le))
		f.close()
		del oe, oi, features, labels, images, imagePaths, f

	else:
		print("This features (" + fE[0] + ") are already generated")


def generateFeatures(outputPath, batchSize, datasetPath, featureExtractors, verbose=False):
	# hp.setrelheap()


	# shuffle the image paths to ensure randomness -- this will help make our
	# training and testing split code more efficient
	imagePaths = list(paths.list_images(datasetPath))
	random.seed(42)
	random.shuffle(imagePaths)

	# determine the set of possible class labels from the image dataset assuming
	# that the images are in {directory}/{filename} structure and create the
	# label encoder
	if verbose:
		print("[INFO] encoding labels...")
	le = LabelEncoder()
	le.fit([p.split("/")[-2] for p in imagePaths])

	# fParams = open("featureExtractors.csv")
	# Parallel(n_jobs=-1)(delayed(extractFeatures)(fE, batchSize, datasetPath, outputPath,datasetP, le, verbose) for fE in featureExtractors)
	for (fE) in featureExtractors:
		# fParams.write(fE[0] + "," + fE[1])
		extractFeatures(fE,batchSize, imagePaths, outputPath, datasetPath, le, verbose)
	# h = hp.heap()
	# print(h)

	del le, imagePaths

def __main__():
	# construct the argument parser and parse the command line arguments
	ap = argparse.ArgumentParser()
	ap.add_argument("-c", "--conf", required=True, help="path to configuration file")
	args = vars(ap.parse_args())
	# load the configuration and label encoder
	conf = Conf(args["conf"])

	outputPath = conf["output_path"]
	datasetPath = conf["dataset_path"]
	featureExtractors = conf["feature_extractors"]
	batchSize = conf["batch_size"]

	# load the configuration and grab all image paths in the dataset
	# generateFeatures(conf,imagePaths,"False")
	generateFeatures(outputPath, batchSize, datasetPath, featureExtractors, False)


if __name__ == "__main__":
    __main__()



from sklearn import metrics



outputPath = "../output"
datasetName= "/dataAugmentation"

auxPath = outputPath + datasetName
filePrediction = auxPath +"/predictionResults.csv"
predictions = []
csvfile = open(filePrediction, "r")
csvfile.readline()
for line in csvfile:
    line = line.split(",")
    predictions.append(int(line[1]))




groundTruthFile = "../test_GroundTruth.csv"
groundTruth = []
csvfile2 = open(groundTruthFile, "r")
csvfile2.readline()
for line in csvfile2:
    line = line.split(",")
    groundTruth.append(int(float(line[1])))

print("Ground Truth")
print (groundTruth)
print("Predictions")
print(predictions)

print(metrics.roc_auc_score(groundTruth, predictions))
print(metrics.accuracy_score(groundTruth,predictions))


# import the necessary packages
#Example python prediction.py -c ../conf/confMelanomaDataAugmentation.conf -i ../pos.jpg -f overfeat -p [-3] -m svm
from __future__ import print_function

from extractor.extractor import Extractor
from utils.conf import Conf
from utils import dataset
from index_features import extractFeatures
from shallowmodels.classificationModelFactory import classificationModelFactory
import numpy as np
import argparse
import cPickle
import h5py
from sklearn.model_selection import RandomizedSearchCV
from sklearn.preprocessing import LabelEncoder
import random
from imutils import paths
import os
import json

def prediction(featExt, classi, imagePath, outputPath, datasetPath):
    # load the configuration, label encoder, and classifier
    print("[INFO] loading model...")
    datasetName = datasetPath[datasetPath.rfind("/"):]
    auxPath = outputPath + datasetName
    factory = classificationModelFactory()

    cPickleFile = auxPath + "/classifier_" + featExt[0] + "_" + classi + ".cpickle"
    if os.path.isfile(cPickleFile):
        labelEncoderPath = auxPath + "/models/le-" + featExt[0] + ".cpickle"
        le = cPickle.loads(open(labelEncoderPath).read())
        model = cPickle.loads(open(cPickleFile).read())

    else: #Aqui es donde deberia hacer la pregunta de que si quiere realmente entrenar ese modelo o el mejor
        print("This is not the best model. Are you sure you want to predict with it?")
        response = raw_input()
        if (response.upper() in ("YES", "Y")):
            files = os.walk(auxPath + "/models")
            listFeatExt = []
            for _, _, file in files:
                for feat in file:
                    feat = feat.split("-")[1].split(".")[0]
                    listFeatExt.append(feat)
            listFeatExt = list(set(listFeatExt))
            if (featExt[0] in listFeatExt):
                labelEncoderPath = auxPath + "/models/le-" + featExt[0] + ".cpickle"
                le = cPickle.loads(open(labelEncoderPath).read())
                featuresPath = auxPath + "/models/features-" + featExt[0] + ".hdf5"
                db = h5py.File(featuresPath)
                split = int(db["image_ids"].shape[0])
                (trainData, trainLabels) = (db["features"][:split], db["image_ids"][:split])
                trainLabels = [le.transform([l.split(":")[0]])[0] for l in trainLabels]
                classifierModel = factory.getClassificationModel(classi)

                model = RandomizedSearchCV(classifierModel.getModel(), param_distributions=classifierModel.getParams(),
                                           n_iter=classifierModel.getNIterations())
                model.fit(trainData, trainLabels)
                f = open(auxPath + "/classifier_" + featExt[0] + "_" + classi + ".cpickle", "w")
                f.write(cPickle.dumps(model))
                f.close()
                db.close()

            else:
                imagePaths = list(paths.list_images(datasetPath))
                random.seed(42)
                random.shuffle(imagePaths)
                le = LabelEncoder()
                le.fit([p.split("/")[-2] for p in imagePaths])
                extractFeatures(featExt, 32, imagePaths, outputPath, datasetPath, le, False)
                labelEncoderPath = auxPath + "/models/le-" + featExt[0] + ".cpickle"
                le = cPickle.loads(open(labelEncoderPath).read())
                featuresPath = auxPath + "/models/features-" + featExt[0] + ".hdf5"
                db = h5py.File(featuresPath)
                split = int(db["image_ids"].shape[0])
                (trainData, trainLabels) = (db["features"][:split], db["image_ids"][:split])
                trainLabels = [le.transform([l.split(":")[0]])[0] for l in trainLabels]
                classifierModel = factory.getClassificationModel(classi)
                model = RandomizedSearchCV(classifierModel.getModel(), param_distributions=classifierModel.getParams(),
                                           n_iter=classifierModel.getNIterations())
                model.fit(trainData, trainLabels)
                f = open(auxPath + "/classifier_" + featExt[0] + "_" + classi + ".cpickle", "w")
                f.write(cPickle.dumps(model))
                f.close()
                db.close()
        else:
            labelEncoderPath = auxPath + "/models/le-" + featExt[0] + ".cpickle"
            le = cPickle.loads(open(labelEncoderPath).read())
            model = cPickle.loads(open(cPickleFile).read())


    filePrediction = open(auxPath+"/predictionResults.csv","a")
    filePrediction.write("image_id, melanoma\n")
    oe = Extractor(featExt)
    imagePaths = list(paths.list_images(imagePath))

    for (i, imPaths) in enumerate(dataset.chunk(imagePaths, 32)):
        (labels, images) = dataset.build_batch(imPaths, featExt[0])

        features = oe.describe(images)
        for (label, vector) in zip(labels, features):
            prediction = model.predict(np.atleast_2d(vector))[0]
            filePrediction.write( str(label) + ", " + str(prediction) + "\r\n")
            prediction = le.inverse_transform(prediction)
            print("[INFO] class predicted for the image {}: {}".format(label, prediction))

    filePrediction.close()


#This method only if you want to execute the prediction with the output of the previous steps
def predictionSimple(imagePath, outputPath, datasetPath):
    datasetName = datasetPath[datasetPath.rfind("/"):]
    auxPath = outputPath + datasetName
    with open(auxPath + "ConfModel.json") as json_file:
        data = json.load(json_file)

    extractor = data[0]['featureExtractor']
    classifier = data[0]['classifierModel']
    prediction(extractor,classifier,imagePath,outputPath,datasetPath)

def __main__():
    # construct the argument parser and parse the command line arguments
    ap = argparse.ArgumentParser()
    ap.add_argument("-c", "--conf", required=True, help="path to configuration file")
    ap.add_argument("-i", "--image", required=True, help="path to the image to predict")
    ap.add_argument("-f", "--feature", required=True, help="feature extractor to predict the class")
    ap.add_argument("-p", "--params", required=False , help="parameters of the feature extractor")
    ap.add_argument("-m", "--classifierModel", required=True, help="model to classify the images")

    args = vars(ap.parse_args())
    conf = Conf(args["conf"])
    outputPath = conf["output_path"]
    featureExtractor = [args["feature"], args["params"]]
    classifier = args["classifierModel"]

    datasetPath = conf["dataset_path"]
    # load the configuration and grab all image paths in the dataset
    # generateFeatures(conf,imagePaths,"False")
    prediction(featureExtractor, classifier, args["image"], outputPath, datasetPath)


if __name__ == "__main__":
    __main__()



# USAGE
# python fullAnalysis.py --conf conf/flowers17.json
# import the necessary packages
from __future__ import print_function
import warnings
warnings.simplefilter(action="ignore", category=FutureWarning)
import time
import argparse
from utils.conf import Conf
from index_features import generateFeatures
from StatisticalComparison import statisticalComparison
from train import train

# suppress any FutureWarning from Theano


def fullAnalysis(config):
    conf = Conf(config)
    verbose = False
    start = time.time()

    generateFeatures(conf["output_path"], conf["batch_size"], conf["dataset_path"], conf["feature_extractors"], verbose)
    statisticalComparison(conf["output_path"], conf["dataset_path"], conf["feature_extractors"],
                          conf["model_classifiers"], conf["measure"], verbose)

    end = time.time()
    train(conf["output_path"], conf["dataset_path"], conf["training_size"])

    print("It has taken " + str(end - start) + " seg to run")

def __main__():
    # construct the argument parser and parse the command line arguments
    ap = argparse.ArgumentParser()
    ap.add_argument("-c", "--conf", required=True, help="path to configuration file")
    args = vars(ap.parse_args())
    # load the configuration and grab all image paths in the dataset
    fullAnalysis(args["conf"])

if __name__ == "__main__":
    __main__()


